-- Teorie nadklíčů pro relace

DROP TABLE IF EXISTS movie;

CREATE TABLE movie (
    title text,
    year integer,
    length integer NOT NULL,
    PRIMARY KEY ( title, year )
); 
-- Film "title" dlouhý "length" minut 
-- byl vytvořený roku "year".

INSERT INTO movie VALUES 
     ( 'The Matrix', 1999, 136 ),
     ( 'The Avengers', 2012, 143 ),
     ( 'The Avengers', 1998, 89 ),
     ( 'American Beauty', 1999, 122 );

TABLE movie;

/*
Kartézský součin relace se sebou samou 
(jen přejmenujeme atributy).
*/
SELECT m1.title AS title1, 
       m1.year AS year1, 
       m1.length AS length1,
       m2.title AS title2, 
       m2.year AS year2,
       m2.length AS length2
FROM   movie AS m1, 
       movie AS m2;

/*
Omezíme se na n-tice, kde title1 = title2 AND year1 = year2:
*/

SELECT m1.title AS title1, 
       m1.year AS year1, 
       m1.length AS length1,
       m2.title AS title2, 
       m2.year AS year2,
       m2.length AS length2
FROM   movie AS m1, 
       movie AS m2
WHERE  m1.title = m2.title
AND    m1.year = m2.year;


/*
Pak se musí rovnat i length1 a length2:
*/
SELECT m1.title AS title1, 
       m1.year AS year1, 
       m1.length AS length1,
       m2.title AS title2, 
       m2.year AS year2,
       m2.length AS length2
FROM   movie AS m1, 
       movie AS m2
WHERE  m1.title = m2.title
AND    m1.year = m2.year
AND NOT ( m1.length = m2.length );    

/*
Výsledek je prázdná relace, 
právě když {title, year} je nadklíčem 
relace movies.

Tedy {title, year} je nadklíčem relace movies.

Ale množina {title} není nadklíčem relace movies:
*/

SELECT m1.title AS title1, 
       m1.year AS year1, 
       m1.length AS length1,
       m2.title AS title2, 
       m2.year AS year2,
       m2.length AS length2
FROM   movie AS m1, 
       movie AS m2
WHERE  m1.title = m2.title  
AND (  NOT ( m1.length = m2.length )
OR  NOT ( m1.year = m2.year ) );